@@ -34,227 +34,6 @@ addSignificance <- function (
 
 }
 
-#' @title assessDesign
-#'
-#' @description This function performs simulation based trial design evaluations for a set of specified dose-response models
-#'
-#' @param n_patients Vector specifying the planned number of patients per dose group. A minimum of 2 patients are required in each group.
-#' @param mods An object of class "Mods" as specified in the DoseFinding package.
-#' @param prior_list A prior_list object specifying the utilized prior for the different dose groups
-#' @param sd A positive value, specification of assumed sd
-#' @param n_sim Number of simulations to be performed
-#' @param alpha_crit_val (Un-adjusted) Critical value to be used for the MCP testing step. Passed to the getCritProb() function for the calculation of adjusted critical values (on the probability scale). Default is 0.05.
-#' @param modeling Boolean variable defining whether the Mod part of Bayesian MCP-Mod will be performed in the assessment. More heavy on resources. Default FALSE.
-#' @param simple Boolean variable defining whether simplified fit will be applied. Passed to the getModelFits function. Default FALSE.
-#' @param avg_fit Boolean variable, defining whether an average fit (based on generalized AIC weights) should be performed in addition to the individual models. Default TRUE.
-#' @param reestimate Boolean variable defining whether critical value should be calculated with re-estimated contrasts (see getCritProb function for more details). Default FALSE
-#' @param contr An object of class 'optContr' as created by the getContr() function. Allows specification of a fixed contrasts matrix. Default NULL
-#' @param dr_means A vector, allows specification of  individual (not model based) assumed effects per dose group. Default NULL
-#' @param delta A numeric value for the threshold Delta for the MED assessment. If NULL, no MED assessment is performed. Default NULL.
-#' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see ?getMED for details. Default NULL.
-#' @param med_selection A string, either "avgFit" or "bestFit", for the method of MED selection. Default "avgFit".
-#'
-#' @return Returns success probabilities for the different assumed dose-response shapes, attributes also includes information around average success rate (across all assumed models) and prior Effective sample size
-#'
-#' @examples
-#' mods <- DoseFinding::Mods(linear      = NULL,
-#'                           emax        = c(0.5, 1.2),
-#'                           exponential = 2,
-#'                           doses       = c(0, 0.5, 2,4, 8),
-#'                           maxEff      = 6)
-#' sd <- 12
-#' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 12), sigma = 2),
-#'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
-#'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
-#'                    DG_3 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.3, s = 11), sigma = 2) ,
-#'                    DG_4 = RBesT::mixnorm(comp1 = c(w = 1, m = 2, s = 13), sigma = 2))
-#' n_patients <- c(40, 60, 60, 60, 60)
-#'
-#' success_probabilities <- assessDesign(
-#'   n_patients  = n_patients,
-#'   mods        = mods,
-#'   prior_list  = prior_list,
-#'   sd          = sd,
-#'   n_sim       = 1e2) # speed up example run time
-#'
-#' success_probabilities
-#'
-#' if (interactive()) { # takes typically > 5 seconds
-#'
-#' # with MED estimation without bootstrapping
-#' # see ?getMED for details
-#'
-#' success_probabilities <- assessDesign(
-#'   n_patients     = n_patients,
-#'   mods           = mods,
-#'   prior_list     = prior_list,
-#'   sd             = sd,
-#'   modeling       = TRUE,
-#'   n_sim          = 10, # speed up example run time
-#'   delta          = 7)
-#'
-#'   success_probabilities
-#'
-#' # with MED estimation with bootstrapping
-#'
-#' success_probabilities <- assessDesign(
-#'   n_patients     = n_patients,
-#'   mods           = mods,
-#'   prior_list     = prior_list,
-#'   sd             = sd,
-#'   modeling       = TRUE,
-#'   n_sim          = 10, # speed up example run time
-#'   delta          = 7,
-#'   evidence_level = 0.8)
-#'
-#'   success_probabilities
-#'
-#' }
-#'
-#' @export
-assessDesign <- function (
-
-  n_patients,
-  mods,
-  prior_list,
-
-  sd,
-
-  n_sim          = 1e3,
-  alpha_crit_val = 0.05,
-  modeling       = FALSE,
-  simple         = TRUE,
-  avg_fit        = TRUE,
-  reestimate     = FALSE,
-
-  contr          = NULL,
-  dr_means       = NULL,
-
-  delta          = NULL,
-  evidence_level = NULL,
-  med_selection  = c("avgFit", "bestFit")
-
-) {
-
-  checkmate::assert_vector(n_patients, len = length(attr(mods, "doses")), any.missing = FALSE)
-  checkmate::assert_double(n_patients, lower = 2, upper = Inf)
-  checkmate::assert_class(mods, classes = "Mods")
-  checkmate::assert_list(prior_list, names = "named", len = length(attr(mods, "doses")), any.missing = FALSE)
-  # sensitive to how DoseFinding labels their attributes for "Mods" class
-  checkmate::assert_integerish(n_sim, lower = 1, upper = Inf)
-  checkmate::assert_double(alpha_crit_val, lower = 0, upper = 1)
-  checkmate::assert_logical(modeling)
-
-  # TODO: check that prior_list has 'sd_tot' attribute, and that it's numeric # this is not applicable at the moment
-
-  modeling <- ifelse(!is.null(delta), TRUE, modeling)
-
-  dose_levels <- attr(mods, "doses")
-
-  data <- simulateData(
-    n_patients  = n_patients,
-    dose_levels = dose_levels,
-    sd          = sd,
-    mods        = mods,
-    n_sim       = n_sim,
-    dr_means    = dr_means)
-
-  model_names <- colnames(data)[-c(1:3)]
-
-  crit_prob_adj <- getCritProb(
-    mods           = mods,
-    dose_levels    = dose_levels,
-    dose_weights   = n_patients,
-    alpha_crit_val = alpha_crit_val)
-
-  if (!reestimate & is.null(contr)) {
-
-    contr <- getContr(
-      mods           = mods,
-      dose_levels    = dose_levels,
-      dose_weights   = n_patients,
-      prior_list     = prior_list)
-
-  }
-
-  eval_design <- lapply(model_names, function (model_name) {
-
-    posterior_list <- getPosterior(
-      data       = getModelData(data, model_name),
-      prior_list = prior_list)
-
-    if (reestimate & is.null(contr)) {
-
-      post_sds <- sapply(posterior_list, function (post) summary(post)[, 2])
-
-      contr <- apply(post_sds, 2, function (post_sd) getContr(
-        mods          = mods,
-        dose_levels   = dose_levels,
-        cov_posterior = diag(post_sd^2)))
-
-    }
-
-    if (modeling) {
-
-      b_mcp_mod <- performBayesianMCPMod(
-        posterior_list = posterior_list,
-        contr          = contr,
-        crit_prob_adj  = crit_prob_adj,
-        simple         = simple,
-        avg_fit        = avg_fit,
-        delta          = delta,
-        evidence_level = evidence_level,
-        med_selection  = med_selection,
-        n_samples      = n_sim)
-
-    } else {
-
-      b_mcp <- performBayesianMCP(
-        posterior_list = posterior_list,
-        contr          = contr,
-        crit_prob_adj  = crit_prob_adj)
-
-    }
-
-  })
-
-  names(eval_design) <- model_names
-
-  avg_success_rate <- mean(sapply(eval_design, function (x) {
-
-    ifelse(identical(modeling, FALSE),
-           attr(x, "successRate"),
-           attr(x$BayesianMCP, "successRate"))
-
-  }))
-  
-  attr(eval_design, "avgSuccessRate") <- avg_success_rate
-
-  if (modeling & !is.null(delta)) {
-
-    avg_med_id_rate <- mean(sapply(eval_design, function (x) {
-
-      mean(attr(x, "MED")[, "med_reached"])
-
-    }))
-    
-    attr(eval_design, "avgMEDIdentificationRate") <- avg_med_id_rate
-
-  }
-
-  attr(eval_design, "placEff")        <- ifelse(test = is.null(dr_means),
-                                                yes  = attr(mods, "placEff"),
-                                                no   = dr_means[1])
-  attr(eval_design, "maxEff")         <- ifelse(test = is.null(dr_means),
-                                                yes  = attr(mods, "maxEff"),
-                                                no   = diff(range(dr_means)))
-  attr(eval_design, "sampleSize")     <- n_patients
-  attr(eval_design, "priorESS")       <- round(getESS(prior_list), 1)
-
-  return (eval_design)
-
-}
-
 BayesMCPi <- function (
 
   posterior_i,
@@ -449,8 +228,11 @@ getContr <- function (
       w      = w
     )
   }
+  
+  attr(contr, "direction") <- attr(mods, "direction")
 
   return(contr)
+  
 }
 
 #' @title getCritProb
@@ -515,6 +297,7 @@ getCritProb <- function (
   ))
 
   return(crit_prob)
+  
 }
 
 getModelSuccesses <- function (b_mcp) {
@@ -546,6 +329,7 @@ getModelSuccesses <- function (b_mcp) {
 #' @param evidence_level A numeric value between 0 and 1 for the evidence level gamma for the MED assessment. Only required for Bayesian MED assessment, see ?getMED for details. Default NULL.
 #' @param med_selection A string, either "avgFit" or "bestFit" based on the lowest gAIC, for the method of MED selection. Default "avgFit".
 #' @param n_samples A numerical for the number of bootstrapped samples in case the Bayesian MED assessment is performed. Default 1e3.
+#' @param probability_scale A boolean to specify if the trial has a continuous or a binary outcome. Setting to TRUE will transform calculations from the logit scale to the probability scale, which can be desirable for a binary outcome. Default FALSE.
 #' @examples
 #' mods <- DoseFinding::Mods(linear      = NULL,
 #'                           emax        = c(0.5, 1.2),
@@ -561,7 +345,7 @@ getModelSuccesses <- function (b_mcp) {
 #'   mods           = mods,
 #'   dose_weights   = c(50, 50, 50, 50, 50), #reflecting the planned sample size
 #'   dose_levels    = dose_levels,
-#'   alpha_crit_val = 0.6) # unreasonable alpha for this example, rather choose 0.05
+#'   alpha_crit_val = 0.6) # unreasonable alpha chosen for this example, rather choose 0.05
 #' prior_list <- list(Ctrl = RBesT::mixnorm(comp1 = c(w = 1, m = 0, s = 5), sigma = 2),
 #'                    DG_1 = RBesT::mixnorm(comp1 = c(w = 1, m = 1, s = 12), sigma = 2),
 #'                    DG_2 = RBesT::mixnorm(comp1 = c(w = 1, m = 1.2, s = 11), sigma = 2) ,
@@ -579,7 +363,7 @@ getModelSuccesses <- function (b_mcp) {
 #'                       contr          = contr_mat,
 #'                       crit_prob_adj  = critVal,
 #'                       simple         = FALSE,
-#'                       delta          = 1)
+#'                       delta          = 1.1)
 #'
 #' @return Bayesian MCP test result as well as modeling result.
 #'
@@ -589,13 +373,14 @@ performBayesianMCPMod <- function (
   posterior_list,
   contr,
   crit_prob_adj,
-  simple = FALSE,
-  avg_fit = TRUE,
+  simple            = FALSE,
+  avg_fit           = TRUE,
 
-  delta          = NULL,
-  evidence_level = NULL,
-  med_selection  = c("avgFit", "bestFit"),
-  n_samples      = 1e3
+  delta             = NULL,
+  evidence_level    = NULL,
+  med_selection     = c("avgFit", "bestFit"),
+  n_samples         = 1e3,
+  probability_scale = FALSE
 
 ) {
 
@@ -616,6 +401,8 @@ performBayesianMCPMod <- function (
 
   med_selection <- match.arg(med_selection, choices = c("avgFit", "bestFit"))
   get_med       <- ifelse(is.null(delta), FALSE, TRUE)
+  
+  checkmate::assert_flag(probability_scale)
 
   if (!is.null(evidence_level)) {
     stopifnot("delta must not be NULL if evidence_level is not NULL" = !is.null(delta))
@@ -636,6 +423,17 @@ performBayesianMCPMod <- function (
     stop ("Argument 'contr' must be of type 'optContr'")
 
   }
+  
+  if (is.null(attr(contr, "direction"))) {
+    
+    attr(model_names, "direction") <- "increasing"
+    message("'contr' does not have attribute 'direction' and is assumed to be 'increasing'")
+    
+  } else {
+    
+    attr(model_names, "direction") <- attr(contr, "direction")
+    
+  }
 
   b_mcp <- performBayesianMCP(
     posterior_list = posterior_list,
@@ -643,12 +441,13 @@ performBayesianMCPMod <- function (
     crit_prob_adj  = crit_prob_adj)
 
   b_mod <- performBayesianMod(
-    b_mcp          = b_mcp,
-    posterior_list = posterior_list,
-    model_names    = model_names,
-    dose_levels    = dose_levels,
-    simple         = simple,
-    avg_fit        = avg_fit)
+    b_mcp             = b_mcp,
+    posterior_list    = posterior_list,
+    model_names       = model_names,
+    dose_levels       = dose_levels,
+    simple            = simple,
+    avg_fit           = avg_fit,
+    probability_scale = probability_scale)
 
   b_mcp_mod <- list(BayesianMCP = b_mcp, Mod = b_mod)
 
@@ -656,25 +455,32 @@ performBayesianMCPMod <- function (
 
     med_info <- t(sapply(b_mcp_mod$Mod, function (model_fits) {
 
-      if (!is.null(model_fits)) {
+      if (!is.null(model_fits)) { # model shape is significant
 
-        if (!is.null(evidence_level)) {
+        if (!is.null(evidence_level)) { # && abs(evidence_level - 0.5) < 1e-9 # in case MED assessment with 0.5 should result in using the modelFit
+          
+          bs_quantile <- ifelse(test = attr(model_fits, "direction") == "increasing",
+                                yes  = 1 - evidence_level,
+                                no   = evidence_level)
 
           bs_quantiles <- getBootstrapQuantiles(
-            model_fits = model_fits,
-            quantiles  = 1 - evidence_level,
-            n_samples  = n_samples)
+            model_fits        = model_fits,
+            quantiles         = bs_quantile,
+            n_samples         = n_samples,
+            probability_scale = probability_scale)
 
           med_info <- getMED(
-            delta          = delta,
-            evidence_level = evidence_level,
-            bs_quantiles   = bs_quantiles)
+            delta             = delta,
+            evidence_level    = evidence_level,
+            bs_quantiles      = bs_quantiles,
+            probability_scale = probability_scale)
 
         } else {
 
           med_info <- getMED(
-            delta      = delta,
-            model_fits = model_fits)
+            delta             = delta,
+            model_fits        = model_fits,
+            probability_scale = probability_scale)
 
         }
 
@@ -778,9 +584,7 @@ performBayesianMCP <- function(
   checkmate::assert_class(crit_prob_adj, "numeric")
   checkmate::assert_numeric(crit_prob_adj, lower = 0, upper = Inf)
 
-  ## TODO: What case is covered by the IF clause?
-  ## Optionally providing custom contrast?
-  ## This was written before the checkmate assertions were included
+  ## What case is covered by the IF clause?
   ## -> if contrasts are re-evaluated, contr becomes a list of contrasts
   if (inherits(contr, "optContr")) {
 
@@ -812,27 +616,22 @@ performBayesianMod <- function (
   model_names,
   dose_levels,
   simple,
-  avg_fit
+  avg_fit,
+  probability_scale = FALSE
 
 ) {
 
-  ## Make parallel processing optional
-  if (requireNamespace("future.apply", quietly = TRUE)) {
-    optPar_lapply <- future.apply::future_lapply
-  } else {
-    optPar_lapply <- lapply
-  }
-
   fits_list <- optPar_lapply(seq_along(posterior_list), function (i) {
 
     if (b_mcp[i, 1]) {
 
       model_fits <- getModelFits(
-        models      = model_names,
-        dose_levels = dose_levels,
-        posterior   = posterior_list[[i]],
-        avg_fit     = avg_fit,
-        simple      = simple)
+        models            = model_names,
+        dose_levels       = dose_levels,
+        posterior         = posterior_list[[i]],
+        avg_fit           = avg_fit,
+        simple            = simple,
+        probability_scale = probability_scale)
 
       sign_models <- b_mcp[i, -c(1, 2)] > attr(b_mcp, "critProbAdj")
       model_fits  <- addSignificance(model_fits, sign_models)
@@ -843,6 +642,40 @@ performBayesianMod <- function (
 
     }
 
-  })
+  },
+  future.seed       = FALSE,
+  future.scheduling = 1,
+  future.packages   = c("DoseFinding", "RBesT", "nloptr"))
+  
+  # requireNamespace("DoseFinding", quietly = TRUE)
+  # requireNamespace("RBesT",       quietly = TRUE)
+  # requireNamespace("nloptr",      quietly = TRUE)
+  # 
+  # 
+  # fits_list <- parallel::mclapply(seq_along(posterior_list), function (i) {
+  #   
+  #   if (b_mcp[i, 1]) {
+  #     
+  #     model_fits <- getModelFits(
+  #       models            = model_names,
+  #       dose_levels       = dose_levels,
+  #       posterior         = posterior_list[[i]],
+  #       avg_fit           = avg_fit,
+  #       simple            = simple,
+  #       probability_scale = probability_scale)
+  #     
+  #     sign_models <- b_mcp[i, -c(1, 2)] > attr(b_mcp, "critProbAdj")
+  #     model_fits  <- addSignificance(model_fits, sign_models)
+  #     
+  #   } else {
+  #     
+  #     NULL
+  #     
+  #   }
+  #   
+  # },
+  # mc.cores       = 32L,
+  # mc.preschedule = TRUE,
+  # mc.set.seed    = FALSE)
 
 }
